@@ -1,5 +1,5 @@
 #
-#  Copyright (C) 2009, 2015, 2016, 2018, 2019
+#  Copyright (C) 2009, 2015, 2016, 2018, 2019, 2020
 #      Smithsonian Astrophysical Observatory
 #
 #
@@ -22,13 +22,12 @@
 A visualization interface to Sherpa
 """
 
-from __future__ import division
-from __future__ import absolute_import
-import numpy
 import logging
 import importlib
 import sys
 
+import numpy
+
 from sherpa.utils import NoNewAttributesAfterInit, erf, SherpaFloat, \
     bool_cast, parallel_map, dataspace1d, histogram1d, get_error_estimates
 from sherpa.utils.err import PlotErr, StatErr, ConfidenceErr
@@ -38,7 +37,8 @@ from sherpa.stats import Likelihood, LeastSq, Chi2XspecVar
 from sherpa import get_config
 from configparser import ConfigParser
 
-warning = logging.getLogger(__name__).warning
+lgr = logging.getLogger(__name__)
+warning = lgr.warning
 
 # TODO: why is this module globally changing the invalid mode of NumPy?
 _ = numpy.seterr(invalid='ignore')
@@ -84,13 +84,15 @@ backend.init()
 plotter = backend
 
 __all__ = ('Plot', 'Contour', 'Point', 'SplitPlot', 'JointPlot',
-           'DataPlot', 'DataContour', 'DelchiPlot', 'ComponentModelPlot',
+           'DataPlot', 'DataHistogramPlot', 'DataContour', 'DelchiPlot', 'ComponentModelPlot',
            'ModelPlot', 'ModelContour', 'FitPlot', 'FitContour',
            'ResidPlot', 'ResidContour', 'RatioPlot', 'RatioContour',
            'IntervalProjection', 'IntervalUncertainty', 'ChisqrPlot',
            'RegionProjection', 'RegionUncertainty', 'ComponentSourcePlot',
            'PSFPlot', 'PSFContour', 'begin', 'end', 'exceptions', 'backend',
-           'SourcePlot', 'SourceContour', 'Histogram', 'plotter')
+           'SourcePlot', 'SourceContour', 'Histogram',
+           'HistogramPlot', 'ModelHistogramPlot', 'SourceHistogramPlot',
+           'plotter')
 
 _stats_noerr = ('cash', 'cstat', 'leastsq', 'wstat')
 
@@ -143,6 +145,68 @@ def _errorbar_warning(stat):
         "used in fits with {}".format(stat.name)
 
 
+def calculate_errors(data, stat, yerrorbars=True):
+    """Calculate errors from the statistics object."""
+
+    if stat is None:
+        return None
+
+    # Do we warn about adding in error values? The logic here isn't
+    # quite the same as the other times _errorbar_warning is used.
+    # This suggests that the code really should be re-worked to
+    # go through a common code path.
+    #
+    # Do we require that self.yerr is always set, even if the
+    # value is not used in a plot? I am going to assume not
+    # (since the existing code will not change self.yerr if
+    # the stat.name is not in _stats_noerr but an exception is
+    # raised by get_yerr).
+    #
+    # This assumes that the error bars calculated by data.to_plot are
+    # over-ridden once a statistic is given. However, how does this
+    # work if the statistic is a Chi2 variant - e.g. Chi2DataVar -
+    # but the user has given explicit errors (i.e. they are not to
+    # be calcualted by the "DataVar" part but used as is). Does
+    # data.get_yerr handle this for us, or are invalid errors
+    # used here? It appears that the correct answers are being
+    # returned, but should we only call data.get_yerr if yerr
+    # is None/empty/whatever is returned by to_plot? This also
+    # holds for the Resid/RatioPlot classes.
+    #
+    # Note that we should probably return a value for yerr if
+    # we can, even if 'yerrorbars' is set to False, so that
+    # downstream users can make use of the value even if the
+    # plot doesn't. This is similar to how labels or xerr
+    # attributes are created even if they don't get used by the
+    # plot.
+    #
+
+    msg = _errorbar_warning(stat)
+    if stat.name in _stats_noerr:
+        if yerrorbars:
+            warning(msg)
+
+        return data.get_yerr(True, Chi2XspecVar.calc_staterror)
+
+    try:
+        return data.get_yerr(True, stat.calc_staterror)
+    except Exception:
+        # TODO: can we report a useful error here?
+        #
+        # It is possible that this is actually an unrelated
+        # error: it's unclear what error class is expected to
+        # be thrown, but likely ValueError as this is raised
+        # by Chi2DataVar when sent values < 0
+        # (note that over time the behavior has changed from
+        #  <= 0 to < 0, but this error message has not been
+        # changed).
+        #
+        if yerrorbars:
+            warning(msg + "\nzeros or negative values found")
+
+        return None
+
+
 def merge_settings(prefs, user):
     """Merge preference and user settings.
 
@@ -456,6 +520,27 @@ class HistogramPlot(Histogram):
                  self.title,
                  self.histo_prefs))
 
+    def _repr_html_(self):
+        """Return a HTML (string) representation of the histogram plot."""
+        return backend.as_html_histogram(self)
+
+    @property
+    def x(self):
+        """Return (xlo + xhi) / 2
+
+        This is intended to make it easier to swap between
+        plot- and histogram-style plots by providing access
+        to an X value.
+        """
+
+        if self.xlo is None or self.xhi is None:
+            return  None
+
+        # As we do not (yet) require NumPy arrays, enforce it.
+        xlo = numpy.asarray(self.xlo)
+        xhi = numpy.asarray(self.xhi)
+        return (xlo + xhi) / 2
+
     def plot(self, overplot=False, clearwindow=True, **kwargs):
         """Plot the data.
 
@@ -474,7 +559,7 @@ class HistogramPlot(Histogram):
            match the names of the keys of the object's
            plot_prefs dictionary.
 
-        See Also        
+        See Also
         --------
         prepare, overplot
 
@@ -485,6 +570,128 @@ class HistogramPlot(Histogram):
                        overplot=overplot, clearwindow=clearwindow, **kwargs)
 
 
+# I think we want slightly different histogram preferences
+# than most (mark by point rather than line).
+#
+def get_data_hist_prefs():
+    """Copy the data prefences to the histogram class"""
+
+    hprefs = backend.get_model_histo_defaults()
+    dprefs = backend.get_data_plot_defaults()
+    for k, v in dprefs.items():
+        if k not in hprefs:
+            continue
+
+        hprefs[k] = v
+
+    return hprefs
+
+
+class DataHistogramPlot(HistogramPlot):
+    """Create 1D histogram plots of data values."""
+
+    histo_prefs = get_data_hist_prefs()
+
+    def __init__(self):
+        self.xerr = None
+        self.yerr = None
+        super().__init__()
+
+    def prepare(self, data, stat=None):
+        """Create the data to plot
+
+        Parameters
+        ----------
+        data
+            The Sherpa data object to display (it is assumed to be
+            one dimensional and represent binned data).
+        stat : optional
+            The Sherpa statistics object to use to add Y error bars
+            if the data has none.
+
+        See Also
+        --------
+        plot
+        """
+
+        # Need a better way of accessing the binning of the data.
+        # Maybe to_plot should return the lo/hi edges as a pair
+        # here.
+        #
+        (_, self.y, self.yerr, self.xerr, self.xlabel,
+         self.ylabel) = data.to_plot()
+
+        self.xlo, self.xhi = data.get_indep(True)
+
+        if stat is not None:
+            yerrorbars = self.histo_prefs.get('yerrorbars', True)
+            self.yerr = calculate_errors(data, stat, yerrorbars)
+
+        self.title = data.name
+
+    def plot(self, overplot=False, clearwindow=True, **kwargs):
+        """Plot the data.
+
+        This will plot the data sent to the prepare method.
+
+        Parameters
+        ----------
+        overplot : bool, optional
+           If ``True`` then add the data to an exsiting plot, otherwise
+           create a new plot.
+        clearwindow : bool, optional
+           Should the existing plot area be cleared before creating this
+           new plot (e.g. for multi-panel plots)?
+        **kwargs
+           These values are passed on to the plot backend, and must
+           match the names of the keys of the object's
+           plot_prefs dictionary.
+
+        See Also
+        --------
+        prepare, overplot
+
+        """
+
+        Histogram.plot(self, self.xlo, self.xhi, self.y,
+                       yerr=self.yerr, title=self.title,
+                       xlabel=self.xlabel, ylabel=self.ylabel,
+                       overplot=overplot, clearwindow=clearwindow, **kwargs)
+
+
+class ModelHistogramPlot(HistogramPlot):
+    """Display a model as a histogram."""
+
+    def __init__(self):
+        super().__init__()
+        self.title = 'Model'
+
+    def prepare(self, data, model, stat=None):
+        """Create the plot.
+
+        The stat parameter is ignored.
+        """
+
+        plot = data.to_plot(yfunc=model)
+        (_, y, _, _, self.xlabel, self.ylabel) = plot
+
+        # taken from get_x from Data1DInt
+        indep = data.get_evaluation_indep(filter=True, model=model)
+
+        self.xlo = indep[0]
+        self.xhi = indep[1]
+        self.y = y[1]
+        assert self.y.size == self.xlo.size
+
+
+class SourceHistogramPlot(ModelHistogramPlot):
+    """Display a source model as a histogram."""
+
+    def __init__(self):
+        super().__init__()
+        self.title = 'Source'
+
+
 class PDFPlot(HistogramPlot):
 
     def __init__(self):
@@ -500,6 +707,10 @@ class PDFPlot(HistogramPlot):
 
         return ('points = %s\n' % (points) + HistogramPlot.__str__(self))
 
+    def _repr_html_(self):
+        """Return a HTML (string) representation of the PDF plot."""
+        return backend.as_html_pdf(self)
+
     def prepare(self, points, bins=12, normed=True, xlabel="x", name="x"):
         """Create the data to plot.
 
@@ -590,6 +801,10 @@ class CDFPlot(Plot):
                  self.title,
                  self.plot_prefs))
 
+    def _repr_html_(self):
+        """Return a HTML (string) representation of the CDF plot."""
+        return backend.as_html_cdf(self)
+
     def prepare(self, points, xlabel="x", name="x"):
         """Create the data to plot.
 
@@ -674,6 +889,10 @@ class LRHistogram(HistogramPlot):
                           'lr = %s' % str(self.lr),
                           HistogramPlot.__str__(self)])
 
+    def _repr_html_(self):
+        """Return a HTML (string) representation of the LRHistogram plot."""
+        return backend.as_html_lr(self)
+
     def prepare(self, ratios, bins, niter, lr, ppp):
         "Create the data to plot"
 
@@ -843,26 +1062,28 @@ class SplitPlot(Plot, Contour):
         self.contour(row, col, plot, *args, **kwargs)
 
 
+# TODO: move logic from sherpa.ui.utils.Session._plot_jointplot
+#       regarding the X axis here
+#
 class JointPlot(SplitPlot):
-    "Multiple plots that share a common axis"
+    """Multiple plots that share a common axis
+
+    This supports two plots, where the top plot is twice as
+    tall as the bottom plot.
+    """
 
     def __init__(self):
         SplitPlot.__init__(self)
 
-    def _clear_window(self, row, col, erase=True):
-        if not self._cleared_window:
-            if erase:
-                backend.clear_window()
-            # FIXME: misuse of kwarg clearaxes for chips backend
-            backend.set_jointplot(row, col, self.rows, self.cols, False)
-            self._cleared_window = True
-        else:
-            # FIXME: misuse of kwarg clearaxes for chips backend
-            backend.set_jointplot(row, col, self.rows, self.cols)
+    def plottop(self, plot, *args, overplot=False, clearwindow=True,
+                **kwargs):
 
-    def plottop(self, plot, *args, **kwargs):
-        clearaxes = kwargs.get('clearwindow', True)
-        self._clear_window(0, 0, clearaxes)
+        create = clearwindow and not overplot
+        if create:
+            self._clear_window()
+
+        backend.set_jointplot(0, 0, self.rows, self.cols,
+                              create=create)
 
         # The code used to check if the plot was an instance of
         # FitPlot, which has been updated to check for the presence
@@ -879,15 +1100,17 @@ class JointPlot(SplitPlot):
                 mplot.xlabel = ''
 
         kwargs['clearwindow'] = False
-        plot.plot(*args, **kwargs)
+        plot.plot(*args, overplot=overplot, **kwargs)
 
-    def plotbot(self, plot, *args, **kwargs):
-        self._clear_window(1, 0)
+    def plotbot(self, plot, *args, overplot=False, **kwargs):
+
+        backend.set_jointplot(1, 0, self.rows, self.cols,
+                              create=False)
 
         # FIXME: terrible hack to remove title from bottom
         plot.title = ''
         kwargs['clearwindow'] = False
-        plot.plot(*args, **kwargs)
+        plot.plot(*args, overplot=overplot, **kwargs)
 
 
 class DataPlot(Plot):
@@ -984,6 +1207,10 @@ class DataPlot(Plot):
                  self.title,
                  self.plot_prefs))
 
+    def _repr_html_(self):
+        """Return a HTML (string) representation of the data plot."""
+        return backend.as_html_data(self)
+
     def prepare(self, data, stat=None):
         """Create the data to plot
 
@@ -1004,62 +1231,9 @@ class DataPlot(Plot):
         (self.x, self.y, self.yerr, self.xerr, self.xlabel,
          self.ylabel) = data.to_plot()
 
-        # Do we warn about adding in error values? The logic here isn't
-        # quite the same as the other times _errorbar_warning is used.
-        # This suggests that the code really should be re-worked to
-        # go through a common code path.
-        #
-        # Do we require that self.yerr is always set, even if the
-        # value is not used in a plot? I am going to assume not
-        # (since the existing code will not change self.yerr if
-        # the stat.name is not in _stats_noerr but an exception is
-        # raised by get_yerr).
-        #
-        # This assumes that the error bars calculated by data.to_plot are
-        # over-ridden once a statistic is given. However, how does this
-        # work if the statistic is a Chi2 variant - e.g. Chi2DataVar -
-        # but the user has given explicit errors (i.e. they are not to
-        # be calcualted by the "DataVar" part but used as is). Does
-        # data.get_yerr handle this for us, or are invalid errors
-        # used here? It appears that the correct answers are being
-        # returned, but should we only call data.get_yerr if yerr
-        # is None/empty/whatever is returned by to_plot? This also
-        # holds for the Resid/RatioPlot classes.
-        #
-        # Note that we should probably return a value for yerr if
-        # we can, even if 'yerrorbars' is set to False, so that
-        # downstream users can make use of the value even if the
-        # plot doesn't. This is similar to how labels or xerr
-        # attributes are created even if they don't get used by the
-        # plot.
-        #
-        try:
-            yerrorbars = self.plot_prefs['yerrorbars']
-        except KeyError:
-            yerrorbars = True
-
         if stat is not None:
-            msg = _errorbar_warning(stat)
-            if stat.name in _stats_noerr:
-                self.yerr = data.get_yerr(True, Chi2XspecVar.calc_staterror)
-                if yerrorbars:
-                    warning(msg)
-            else:
-                try:
-                    self.yerr = data.get_yerr(True, stat.calc_staterror)
-                except Exception:
-                    # TODO: can we report a useful error here?
-                    #
-                    # It is possible that this is actually an unrelated
-                    # error: it's unclear what error class is expected to
-                    # be thrown, but likely ValueError as this is raised
-                    # by Chi2DataVar when sent values < 0
-                    # (note that over time the behavior has changed from
-                    #  <= 0 to < 0, but this error message has not been
-                    # changed).
-                    #
-                    if yerrorbars:
-                        warning(msg + "\nzeros or negative values found")
+            yerrorbars = self.plot_prefs.get('yerrorbars', True)
+            self.yerr = calculate_errors(data, stat, yerrorbars)
 
         self.title = data.name
 
@@ -1237,6 +1411,10 @@ class DataContour(Contour):
                  self.levels,
                  self.contour_prefs))
 
+    def _repr_html_(self):
+        """Return a HTML (string) representation of the contour plot."""
+        return backend.as_html_datacontour(self)
+
     def prepare(self, data, stat=None):
         (self.x0, self.x1, self.y, self.xlabel,
          self.ylabel) = data.to_contour()
@@ -1356,6 +1534,10 @@ class ModelPlot(Plot):
                  self.title,
                  self.plot_prefs))
 
+    def _repr_html_(self):
+        """Return a HTML (string) representation of the model plot."""
+        return backend.as_html_model(self)
+
     def prepare(self, data, model, stat=None):
         """Create the data to plot
 
@@ -1415,6 +1597,16 @@ class ComponentModelPlot(ModelPlot):
         self.title = 'Model component: %s' % model.name
 
 
+class ComponentModelHistogramPlot(ModelHistogramPlot):
+
+    # Is this the correct setting?
+    plot_prefs = backend.get_component_plot_defaults()
+
+    def prepare(self, data, model, stat=None):
+        super().prepare(data, model, stat)
+        self.title = 'Model component: {}'.format(model.name)
+
+
 class ComponentTemplateModelPlot(ComponentModelPlot):
 
     def prepare(self, data, model, stat=None):
@@ -1461,6 +1653,27 @@ class ComponentSourcePlot(SourcePlot):
         self.title = 'Source model component: {}'.format(model.name)
 
 
+class ComponentSourceHistogramPlot(SourceHistogramPlot):
+
+    # Is this the correct setting?
+    plot_prefs = backend.get_component_plot_defaults()
+
+    def prepare(self, data, model, stat=None):
+
+        plot = data.to_component_plot(yfunc=model)
+        (_, y, _, _, self.xlabel, self.ylabel) = plot
+
+        # taken from get_x from Data1DInt
+        indep = data.get_evaluation_indep(filter=True, model=model)
+
+        self.xlo = indep[0]
+        self.xhi = indep[1]
+        self.y = y[1]
+        assert self.y.size == self.xlo.size
+
+        self.title = 'Source model component: {}'.format(model.name)
+
+
 class ComponentTemplateSourcePlot(ComponentSourcePlot):
 
     def prepare(self, data, model, stat=None):
@@ -1550,6 +1763,10 @@ class ModelContour(Contour):
                  self.levels,
                  self.contour_prefs))
 
+    def _repr_html_(self):
+        """Return a HTML (string) representation of the model contour plot."""
+        return backend.as_html_modelcontour(self)
+
     def prepare(self, data, model, stat):
         (self.x0, self.x1, self.y, self.xlabel,
          self.ylabel) = data.to_contour(yfunc=model)
@@ -1635,6 +1852,10 @@ class FitPlot(Plot):
                  model_title,
                  self.modelplot))
 
+    def _repr_html_(self):
+        """Return a HTML (string) representation of the fit plot."""
+        return backend.as_html_fit(self)
+
     def prepare(self, dataplot, modelplot):
         """Create the data to plot
 
@@ -1712,6 +1933,10 @@ class FitContour(Contour):
                  model_title,
                  self.modelcontour))
 
+    def _repr_html_(self):
+        """Return a HTML (string) representation of the fit contour plot."""
+        return backend.as_html_fitcontour(self)
+
     def prepare(self, datacontour, modelcontour):
         self.datacontour = datacontour
         self.modelcontour = modelcontour
@@ -1743,6 +1968,11 @@ class DelchiPlot(ModelPlot):
     xlabel, ylabel, title : str
        Plot labels.
 
+    Notes
+    -----
+    The ylog setting is ignored, whether given as a preference or
+    a keyword argument, so the Y axis is always drawn with a
+    linear scale.
     """
 
     plot_prefs = backend.get_resid_plot_defaults()
@@ -1765,6 +1995,8 @@ class DelchiPlot(ModelPlot):
         self.title = _make_title('Sigma Residuals', data.name)
 
     def plot(self, overplot=False, clearwindow=True, **kwargs):
+        self.plot_prefs['ylog'] = False
+        kwargs.pop('ylog', True)
         Plot.plot(self, self.x, self.y, yerr=self.yerr, xerr=self.xerr,
                   title=self.title, xlabel=self.xlabel, ylabel=self.ylabel,
                   overplot=overplot, clearwindow=clearwindow, **kwargs)
@@ -1836,6 +2068,11 @@ class ResidPlot(ModelPlot):
     xlabel, ylabel, title : str
        Plot labels.
 
+    Notes
+    -----
+    The ylog setting is ignored, whether given as a preference or
+    a keyword argument, so the Y axis is always drawn with a
+    linear scale.
     """
 
     plot_prefs = backend.get_resid_plot_defaults()
@@ -1873,6 +2110,8 @@ class ResidPlot(ModelPlot):
         self.title = _make_title('Residuals', data.name)
 
     def plot(self, overplot=False, clearwindow=True, **kwargs):
+        self.plot_prefs['ylog'] = False
+        kwargs.pop('ylog', True)
         Plot.plot(self, self.x, self.y, yerr=self.yerr, xerr=self.xerr,
                   title=self.title, xlabel=self.xlabel, ylabel=self.ylabel,
                   overplot=overplot, clearwindow=clearwindow, **kwargs)
@@ -1920,6 +2159,11 @@ class RatioPlot(ModelPlot):
     xlabel, ylabel, title : str
        Plot labels.
 
+    Notes
+    -----
+    The ylog setting is ignored, whether given as a preference or
+    a keyword argument, so the Y axis is always drawn with a
+    linear scale.
     """
 
     plot_prefs = backend.get_ratio_plot_defaults()
@@ -1957,6 +2201,8 @@ class RatioPlot(ModelPlot):
         self.title = _make_title('Ratio of Data to Model', data.name)
 
     def plot(self, overplot=False, clearwindow=True, **kwargs):
+        self.plot_prefs['ylog'] = False
+        kwargs.pop('ylog', True)
         Plot.plot(self, self.x, self.y, yerr=self.yerr, xerr=self.xerr,
                   title=self.title, xlabel=self.xlabel, ylabel=self.ylabel,
                   overplot=overplot, clearwindow=clearwindow, **kwargs)
@@ -2045,6 +2291,10 @@ class Confidence1D(DataPlot):
                  self.fac,
                  self.log))
 
+    def _repr_html_(self):
+        """Return a HTML (string) representation of the confidence 1D plot."""
+        return backend.as_html_contour1d(self)
+
     def prepare(self, min=None, max=None, nloop=20,
                 delv=None, fac=1, log=False, numcores=None):
         """Set the data to plot.
@@ -2245,6 +2495,10 @@ class Confidence2D(DataContour, Point):
                  self.parval1,
                  self.levels))
 
+    def _repr_html_(self):
+        """Return a HTML (string) representation of the confidence 2D plot."""
+        return backend.as_html_contour2d(self)
+
     def prepare(self, min=None, max=None, nloop=(10, 10),
                 delv=None, fac=4, log=(False, False),
                 sigma=(1, 2, 3), levels=None, numcores=None):
